<?php

if (!defined('ABSPATH')) {
    exit;
}

class TransactionCloudHttp
{
    private const CONNECTION_TIMEOUT = 5;
    private const TIMEOUT = 5;

    protected $headers;

    /**
     * Performs request of given type to desired URL with given data
     *
     * @param string $url
     * @param array $data
     *
     * @throws TransactionCloudNetworkException when there is no response from server
     *
     * @return TransactionCloudHttpResponse
     */
    public function doRequest($url, $data = null)
    {
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $this->getHeaders());
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, self::CONNECTION_TIMEOUT);
        curl_setopt($ch, CURLOPT_TIMEOUT, self::TIMEOUT);

        if ($data) {
            $encodedData = json_encode($data);
            curl_setopt($ch, CURLOPT_POST, true);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $encodedData);
        }

        $response = curl_exec($ch);
        $responseCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

        if($response === false) {
            throw new TransactionCloudNoResponseException(curl_error($ch));
        }

        curl_close($ch);

        if ($responseCode != 200) {
            throw new TransactionCloudBadResponseException(sprintf(
                'Incorrect API response with status %d',
                $responseCode
            ), $responseCode);
        }

        return $response;
    }

    public function getHeaders()
    {
        return $this->headers;
    }
}
